<?php
/**
 * Created by PhpStorm.
 * User: TREULIAUX
 * Date: 12/12/2016
 * Time: 18:10
 */

/**
 * @file
 * Contains \Drupal\media_upload\Form\MediaUploadConfigurationForm.
 */

namespace Drupal\media_upload\Form;

use Drupal\Core\Form\ConfigFormBase;
use Symfony\Component\HttpFoundation\Request;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Form;

/**
 * Contribute form.
 */
class MediaUploadConfigurationForm extends ConfigFormBase {
  const COMPLETE_FILE_NAME = 0;
  const FILE_NAME = 1;
  const TAG_NAME = 2;
  const EXT_NAME = 3;
  private $_allowedImgExt = ['png', 'jpg', 'jpeg', 'gif'];
  private $_allowedDocExt = [
    'pdf',
    'docx',
    'xlsx',
    'pptx',
    'doc',
    'xls',
    'ppt',
    'zip',
    'csv'
  ];
  private $_allowedVidExt = ['mp4'];
  private $_mediaBundle   = NULL;
  private $_imgMaxSize    = 2 * 1024 * 1024;
  private $_docMaxSize    = 20 * 1024 * 1024;
  private $_vidMaxSize    = 50 * 1024 * 1024;
  private $_totalMaxSize  = 50 * 1024 * 1024;

  public function __construct(
    \Drupal\Core\Config\ConfigFactoryInterface $config_factory
  ) {
    parent::__construct($config_factory);
    $this->_mediaBundle = [
      'image'    => $this->_allowedImgExt,
      'document' => $this->_allowedDocExt,
      'video'    => $this->_allowedVidExt
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {

    return 'mediaupload_settings';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(
    array $form,
    FormStateInterface $form_state,
    Request $request = NULL
  ) {

    $config = $this->config('media_upload.settings');
    $form['bundles'] = array(
      '#type'          => 'checkboxes',
      '#title'         => $this->t('Accepted medias'),
      '#options'       => [
        'video'    => $this->t('Videos'),
        'image'    => $this->t('Images'),
        'document' => $this->t('Documents')
      ],
      '#default_value' => $config->get('bundles'),
    );
    $form['total_size'] = array(
      '#type'          => 'textfield',
      '#title'         => $this->t('Max total size'),
      '#description'   => $this->t('Size in Mb. Keep blank or set to 0 to disable limit.'),
      '#default_value' => $config->get('total_size'),
    );
    $form['single_size'] = array(
      '#type'          => 'textfield',
      '#title'         => $this->t('Max single file size'),
      '#description'   => $this->t('Size in Mb. Keep blank or set to 0 to disable limit.'),
      '#default_value' => $config->get('single_size'),
    );

    $form['videos'] = array(
      '#type'   => 'fieldset',
      '#title'  => $this->t('Videos parameters'),
      '#states' => array(
        'invisible' => array(
          ':input[name="bundles[video]"]' => array('checked' => FALSE),
        ),
        'disabled'  => array(
          ':input[name="bundles[video]"]' => array('checked' => FALSE),
        )
      )
    );
    $form['videos']['video_ext'] = array(
      '#type'          => 'textfield',
      '#title'         => $this->t('Allowed videos extensions'),
      '#description'   => $this->t('Accepted extensions for videos media; comma-separated.') . ' Ex: mp4, avi, ...',
      '#default_value' => $config->get('video_ext'),
      '#states'        => array(
        'required' => array(
          ':input[name="bundles[video]"]' => array('checked' => TRUE),
        ),
        'disabled' => array(
          ':input[name="bundles[video]"]' => array('checked' => FALSE),
        )
      )
    );
    $form['videos']['video_size'] = array(
      '#type'          => 'textfield',
      '#title'         => $this->t('Max video size'),
      '#description'   => $this->t('Size in Mb. Keep blank or set to 0 to disable limit.'),
      '#default_value' => $config->get('video_size'),
    );

    $form['images'] = array(
      '#type'   => 'fieldset',
      '#title'  => $this->t('Images parameters'),
      '#states' => array(
        'invisible' => array(
          ':input[name="bundles[image]"]' => array('checked' => FALSE),
        )
      )
    );
    $form['images']['image_ext'] = array(
      '#type'          => 'textfield',
      '#title'         => $this->t('Allowed images extensions'),
      '#description'   => $this->t('Accepted extensions for videos media; comma-separated.') . ' Ex: jpg, png, ...',
      '#default_value' => $config->get('image_ext'),
      '#states'        => array(
        'required' => array(
          ':input[name="bundles[image]"]' => array('checked' => TRUE),
        ),
        'disabled' => array(
          ':input[name="bundles[image]"]' => array('checked' => FALSE),
        )
      )
    );
    $form['images']['image_size'] = array(
      '#type'          => 'textfield',
      '#title'         => $this->t('Max image size'),
      '#description'   => $this->t('Size in Mb. Keep blank or set to 0 to disable limit.'),
      '#default_value' => $config->get('image_size'),
    );

    $form['documents'] = array(
      '#type'   => 'fieldset',
      '#title'  => $this->t('Documents parameters'),
      '#states' => array(
        'invisible' => array(
          ':input[name="bundles[document]"]' => array('checked' => FALSE),
        )
      )
    );
    $form['documents']['document_ext'] = array(
      '#type'          => 'textfield',
      '#title'         => $this->t('Allowed documents extensions'),
      '#description'   => $this->t('Accepted extensions for videos media; comma-separated.') . ' Ex: docx, pdf, ...',
      '#default_value' => $config->get('document_ext'),
      '#states'        => array(
        'required' => array(
          ':input[name="bundles[document]"]' => array('checked' => TRUE),
        ),
        'disabled' => array(
          ':input[name="bundles[document]"]' => array('checked' => FALSE),
        )
      )
    );
    $form['documents']['document_size'] = array(
      '#type'          => 'textfield',
      '#title'         => $this->t('Max document size'),
      '#description'   => $this->t('Size in Mb. Keep blank or set to 0 to disable limit.'),
      '#default_value' => $config->get('document_size'),
    );
    return parent::buildForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function validateForm(array &$form, FormStateInterface $form_state) {
    $this->checkExtensions('video_ext', $form_state);
    $this->checkExtensions('image_ext', $form_state);
    $this->checkExtensions('document_ext', $form_state);
    $this->isInt('total_size', $form_state);
    $this->isInt('single_size', $form_state);
    $this->isInt('video_size', $form_state);
    $this->isInt('image_size', $form_state);
    $this->isInt('document_size', $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $values = $form_state->getValues();
    $config = $this->config('media_upload.settings');

    $config->set('bundles', $values['bundles']);
    $config->set('total_size', $values['total_size']);
    $config->set('single_size', $values['single_size']);

    if (!empty($values['bundles']['video'])) {
      $config->set('video_ext', preg_replace('/\s+/', '', $values['video_ext']));
      $config->set('video_size', $values['video_size']);
    }
    else {
      $config->set('video_ext', '');
      $config->set('video_size', '');
    }

    if (!empty($values['bundles']['image'])) {
      $config->set('image_ext', preg_replace('/\s+/', '', $values['image_ext']));
      $config->set('image_size', $values['image_size']);
    }
    else {
      $config->set('image_ext', '');
      $config->set('image_size', '');
    }

    if (!empty($values['bundles']['document'])) {
      $config->set('document_ext', preg_replace('/\s+/', '', $values['document_ext']));
      $config->set('document_size', $values['document_size']);
    }
    else {
      $config->set('document_ext', '');
      $config->set('document_size', '');
    }

    $config->save();
    drupal_set_message('Configuration done.');
  }

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames() {

    return ['media_upload.settings'];
  }

  private function checkExtensions($fieldName, FormStateInterface $form_state) {
    $extensions = $form_state->getValue($fieldName);
    $extensions = preg_replace('/\s+/', '', $extensions);
    if (!preg_match('/(^$)|(^([a-z0-9]+)(,\s*[a-z0-9]+)*$)/', $extensions)) {
      $form_state->setErrorByName($fieldName, $this->t('The extensions values must be comma-separated'));
    }
  }

  private function isInt($fieldName, FormStateInterface $form_state) {
    $val = $form_state->getValue($fieldName);
    if (empty($val)) {
      $val = 0;
    }
    if (!is_numeric($val)) {
      $form_state->setErrorByName($fieldName, $this->t('The size must be a positive interger value'));
    }
    else if (intval($val) < 0) {
      $form_state->setErrorByName($fieldName, $this->t('The size must be a positive interger value'));
    }
  }
}
