<?php
/**
 * Created by PhpStorm.
 * User: TREULIAUX
 * Date: 12/12/2016
 * Time: 18:10
 */

/**
 * @file
 * Contains \Drupal\media_upload\Form\UploadMediaForm.
 */

namespace Drupal\media_upload\Form;

use Drupal\Core\Form\ConfigFormBase;
use Drupal\media_entity\Entity\Media;
use Drupal\Core\Form\FormStateInterface;
use Drupal\taxonomy\Entity\Term;
use Drupal\Core\Form;

/**
 * Contribute form.
 */
class UploadMediaForm extends ConfigFormBase {
  const COMPLETE_FILE_NAME = 0;
  const FILE_NAME = 1;
  const TAG_NAME = 2;
  const EXT_NAME = 3;
  private $_allowedImgExt     = ['png', 'jpg', 'jpeg', 'gif'];
  private $_allowedDocExt     = [
    'pdf',
    'docx',
    'xlsx',
    'pptx',
    'doc',
    'xls',
    'ppt',
    'zip',
    'csv'
  ];
  private $_allowedVidExt     = ['mp4'];
  private $_mediaBundle       = NULL;
  private $_imgMaxSize        = 2 * 1024 * 1024;
  private $_docMaxSize        = 20 * 1024 * 1024;
  private $_vidMaxSize        = 50 * 1024 * 1024;
  private $_totalMaxSize      = 50 * 1024 * 1024;
  private $_singleFileMaxSize = 50 * 1024 * 1024;

  public function __construct(
    \Drupal\Core\Config\ConfigFactoryInterface $config_factory
  ) {
    parent::__construct($config_factory);
    $config = $this->config('media_upload.settings');
    foreach ($config->get('bundles') as $bundleName => $dummy) {
      if (($values = $config->get($bundleName . '_ext')) != NULL) {
        $this->_mediaBundle[$bundleName] = explode(',', $config->get($bundleName . '_ext'));
      }
    }
    $this->_allowedImgExt = explode(',', $config->get('image_ext'));
    $this->_allowedDocExt = explode(',', $config->get('document_ext'));
    $this->_allowedVidExt = explode(',', $config->get('video_ext'));
    $this->_imgMaxSize = $config->get('image_size') * 1024 * 1024;
    $this->_docMaxSize = $config->get('document_size') * 1024 * 1024;
    $this->_vidMaxSize = $config->get('video_size') * 1024 * 1024;
    $this->_totalMaxSize = $config->get('total_size') * 1024 * 1024;
    $this->_singleFileMaxSize = $config->get('single_size') * 1;
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {

    return 'mediaupload_form';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    if (empty($this->_mediaBundle)) {
      return drupal_set_message('No media bundle is enabled in module configuration', 'warning');
    }
    $form = parent::buildForm($form, $form_state);

    $form['dropzonejs'] = array(
      '#type'                 => 'dropzonejs',
      '#title'                => $this->t('Dropzone'),
      '#required'             => TRUE,
      '#dropzone_description' => 'Cliquez ou déposez vos fichiers ici',
      '#max_filesize'         => $this->_singleFileMaxSize === 0 ? '' : strval($this->_singleFileMaxSize) . 'M',
      '#extensions'           => trim(implode(' ', $this->_allowedDocExt) . ' ' . implode(' ', $this->_allowedImgExt) . ' ' . implode(' ', $this->_allowedVidExt)),
    );

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    try {
      $values = $form_state->getValues();
      if (!empty($values['dropzonejs']) && !empty($values['dropzonejs']['uploaded_files'])) {
        $files = $values['dropzonejs']['uploaded_files'];

        if (!$this->checkTotalSize($files)) {
          \Drupal::logger('mediaupload')
                 ->error($this->t('La taille totale des fichiers uploadés est dépassée') . '(' . ($this->_totalMaxSize / 1024 / 1024) . 'Mo)');
          return drupal_set_message($this->t('La taille totale des fichiers uploadés est dépassée') . ' (' . ($this->_totalMaxSize / 1024 / 1024) . 'Mo)', 'error');
        }

        foreach ($files as $file) {
          $fileInfo = [];
          if (preg_match('/(^[\w-_\.]+)#([\w-_\.]+)\.([a-zA-Z]{3,4})/', $file['filename'], $fileInfo) === 1) {
            if (($bundle = $this->getBundleForFile($fileInfo[self::EXT_NAME])) !== FALSE) {
              if ($this->checkFileSize($file['path'], $bundle)) {
                $data = file_get_contents($file['path']);
                if (file_exists('public://' . $file['filename'])) {
                  \Drupal::logger('mediaupload')
                         ->notice($this->t('Le fichier "') . $file['filename'] . $this->t('" existait déjà - il a été remplacé'));
                  drupal_set_message($this->t('Le fichier "') . $file['filename'] . $this->t('" existait déjà - il a été remplacé'));
                }
                $file_entity = file_save_data($data, 'public://' . $file['filename'], FILE_EXISTS_REPLACE);
                $media = Media::create($this->getFieldsForMedia($fileInfo, $bundle, $file_entity));
                $media->save();
              }
              else {
                \Drupal::logger('mediaupload')
                       ->warning($this->t('La taille maximum du fichier "') . $file['filename'] . $this->t('" est dépassée'));
                drupal_set_message($this->t('La taille maximum du fichier "') . $file['filename'] . $this->t('" est dépassée'), 'warning');
              }
            }
            else {
              \Drupal::logger('mediaupload')
                     ->error($this->t('L\'extension du fichier "') . $file['filename'] . $this->t('" n\'est pas autorisée'));
              drupal_set_message($this->t('L\'extension du fichier "') . $file['filename'] . $this->t('" n\'est pas autorisée'), 'error');
            }
          }
          else {
            \Drupal::logger('mediaupload')
                   ->warning($this->t('Le nom du fichier "') . $file['filename'] . $this->t('" n\'est pas conforme'));
            drupal_set_message($this->t('Le nom du fichier "') . $file['filename'] . $this->t('" n\'est pas conforme'), 'warning');
          }
        }
        \Drupal::logger('mediaupload')
               ->info($this->t('Les documents ont été pris en compte'));
        return drupal_set_message($this->t('Les documents ont été pris en compte'));
      }
      \Drupal::logger('mediaupload')
             ->warning($this->t('Aucun document uploadé'));
      return drupal_set_message($this->t('Aucun document uploadé'), 'warning');
    } catch (\Exception $e) {
      \Drupal::logger('mediaupload')
             ->critical($e->getMessage());
      return drupal_set_message($e->getMessage(), 'error');
    }
  }

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames() {

    return ['media_upload.settings'];
  }

  /**
   * @param array $files La liste des fichiers à importer
   *
   * @return bool
   */
  private function checkTotalSize(array $files) {
    if ($this->_totalMaxSize === 0) {
      return TRUE;
    }
    $totalSize = 0;
    foreach ($files as $file) {
      $totalSize += stat($file['path'])[7];
    }
    if ($totalSize > $this->_totalMaxSize) {
      return FALSE;
    }
    return TRUE;
  }

  /**
   * Vérifie la taille d'un fichier
   *
   * @param $filePath string Chemin d'accès au fichier
   * @param $bundle   string Nom du bundle où sera stocké le média
   *
   * @return bool
   */
  private function checkFileSize($filePath, $bundle) {
    $size = stat($filePath)[7];
    switch ($bundle) {
      case 'image' :
        return ($this->_imgMaxSize === 0 || $size <= $this->_imgMaxSize) ? TRUE : FALSE;
        break;
      case 'video' :
        return ($this->_vidMaxSize === 0 || $size <= $this->_vidMaxSize) ? TRUE : FALSE;
        break;
      case 'document' :
        return ($this->_docMaxSize === 0 || $size <= $this->_docMaxSize) ? TRUE : FALSE;
        break;
      default :
        return FALSE;
    }
  }

  /**
   * Retrieves a tid for use from taxonomy.
   *
   * @param $term_value string Le label taxo
   * @param $vocabulary string Le nom machine du vocabulaire utilisé
   *
   * @return int|mixed|null|string
   */
  private function get_vocabulary_term($term_value, $vocabulary = NULL) {
    if ($terms = taxonomy_term_load_multiple_by_name($term_value, $vocabulary)) {
      $term = reset($terms);
    }
    else {
      $term = Term::create([
                             'name' => $term_value,
                             'vid'  => $vocabulary,
                           ]);
      $term->save();
    }
    return $term->id();
  }

  /**
   * @param $fileExt string L'extension du fichier
   *
   * @return string|FALSE
   */
  private function getBundleForFile($fileExt) {
    $bundle = FALSE;
    foreach ($this->_mediaBundle as $bundleName => $acceptedFormats) {
      if (in_array($fileExt, $acceptedFormats)) {
        $bundle = $bundleName;
        break;
      }
    }
    return $bundle;
  }

  /**
   * @param $fileInfo    array Les infos extraites du nom du fichier
   * @param $bundle      string Le media bundle dans lequel placer le média
   * @param $file_entity \Drupal\file\FileInterface Le file_entity du fichier
   *
   * @return array
   */
  private function getFieldsForMedia(
    array $fileInfo,
    $bundle,
    $file_entity
  ): array {
    $taxonomies = [
      $this->get_vocabulary_term('masse', 'categorie_de_media'),
      $this->get_vocabulary_term($fileInfo[self::TAG_NAME], 'categorie_de_media')
    ];

    $fieldFile = "field_$bundle";
    $fields = [
      'bundle'           => $bundle,
      'name'             => $fileInfo[self::FILE_NAME],
      $fieldFile         => [
        'target_id' => $file_entity->id(),
        'alt'       => $fileInfo[self::FILE_NAME],
        'title'     => $fileInfo[self::FILE_NAME]
      ],
      'field_categories' => $taxonomies
    ];
    return $fields;
  }
}
